@@ -1,12 +1,17 @@
 #![feature(test)]
 
 extern crate test;
-use bincode::{deserialize, serialize};
-use solana_sdk::instruction::{AccountMeta, Instruction};
-use solana_sdk::message::Message;
-use solana_sdk::pubkey;
-use solana_sdk::sysvar::instructions;
-use test::Bencher;
+use {
+    bincode::{deserialize, serialize},
+    solana_sdk::{
+        instruction::{AccountMeta, Instruction},
+        message::{Message, SanitizedMessage},
+        pubkey::{self, Pubkey},
+        sysvar::instructions,
+    },
+    std::convert::TryFrom,
+    test::Bencher,
+};
 
 fn make_instructions() -> Vec<Instruction> {
     let meta = AccountMeta::new(pubkey::new_rand(), false);
@@ -27,7 +32,9 @@ fn bench_bincode_instruction_serialize(b: &mut Bencher) {
 #[bench]
 fn bench_manual_instruction_serialize(b: &mut Bencher) {
     let instructions = make_instructions();
-    let message = Message::new(&instructions, None);
+    let message =
+        SanitizedMessage::try_from(Message::new(&instructions, Some(&Pubkey::new_unique())))
+            .unwrap();
     b.iter(|| {
         test::black_box(message.serialize_instructions(DEMOTE_PROGRAM_WRITE_LOCKS));
     });
@@ -45,7 +52,9 @@ fn bench_bincode_instruction_deserialize(b: &mut Bencher) {
 #[bench]
 fn bench_manual_instruction_deserialize(b: &mut Bencher) {
     let instructions = make_instructions();
-    let message = Message::new(&instructions, None);
+    let message =
+        SanitizedMessage::try_from(Message::new(&instructions, Some(&Pubkey::new_unique())))
+            .unwrap();
     let serialized = message.serialize_instructions(DEMOTE_PROGRAM_WRITE_LOCKS);
     b.iter(|| {
         for i in 0..instructions.len() {
@@ -58,7 +67,9 @@ fn bench_manual_instruction_deserialize(b: &mut Bencher) {
 #[bench]
 fn bench_manual_instruction_deserialize_single(b: &mut Bencher) {
     let instructions = make_instructions();
-    let message = Message::new(&instructions, None);
+    let message =
+        SanitizedMessage::try_from(Message::new(&instructions, Some(&Pubkey::new_unique())))
+            .unwrap();
     let serialized = message.serialize_instructions(DEMOTE_PROGRAM_WRITE_LOCKS);
     b.iter(|| {
         #[allow(deprecated)]
